package com.xzzn.ems.service.impl;

import com.xzzn.ems.domain.vo.DateSearchRequest;
import com.xzzn.ems.domain.vo.ElectricDataInfoVo;
import com.xzzn.ems.domain.vo.SiteMonitorDataVo;
import com.xzzn.ems.mapper.EmsPcsDataMapper;
import com.xzzn.ems.service.IEmsStatsReportService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

/**
 * 统计报表数据Service业务层处理
 *
 * @author xzzn
 * @date 2025-06-29
 */
@Service
public class EmsStatsReportServiceImpl implements IEmsStatsReportService
{

    @Autowired
    private EmsPcsDataMapper emsPcsDataMapper;

    @Override
    public ElectricDataInfoVo getElectricDataResult(DateSearchRequest requestVo) {
        ElectricDataInfoVo electricDataInfoVo = new ElectricDataInfoVo();

        // 根据时间获取每天的充放电量
        List<SiteMonitorDataVo> dataList = emsPcsDataMapper.getPcsDataByDate(requestVo);
        if (!CollectionUtils.isEmpty(dataList)){
            // 总充、总放、效率
            BigDecimal totalDischarge = new BigDecimal(0);
            BigDecimal totalCharge = new BigDecimal(0);
            BigDecimal efficiency = new BigDecimal(0);

            for (SiteMonitorDataVo siteMonitorDataVo : dataList) {
                totalDischarge = totalDischarge.add(siteMonitorDataVo.getDisChargedCap() == null ? BigDecimal.ZERO : siteMonitorDataVo.getDisChargedCap());
                totalCharge = totalCharge.add(siteMonitorDataVo.getChargedCap() ==  null ? BigDecimal.ZERO : siteMonitorDataVo.getChargedCap());

                // 计算单天的效率
                BigDecimal dailyEfficiency = new BigDecimal(0);
                if ( siteMonitorDataVo.getDisChargedCap().compareTo(BigDecimal.ZERO)>0){
                    dailyEfficiency = siteMonitorDataVo.getDisChargedCap().divide(siteMonitorDataVo.getChargedCap(), 2, RoundingMode.HALF_UP);
                }
                siteMonitorDataVo.setDailyEfficiency(dailyEfficiency);
            }
            if ( totalCharge.compareTo(BigDecimal.ZERO)>0){
                efficiency = totalDischarge.divide(totalCharge, 2, RoundingMode.HALF_UP);
            }
            electricDataInfoVo.setSevenDayDisChargeStats(dataList);
            electricDataInfoVo.setTotalDisChargedCap(totalDischarge);
            electricDataInfoVo.setTotalChargedCap(totalCharge);
            electricDataInfoVo.setEfficiency(efficiency);
        }
        return electricDataInfoVo;
    }
}
